# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinFormHandler Action

    @copyright: 2012 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import do_show
from MoinMoin import config
from MoinForms import *

Dependencies = ['pages']

class MoinFormHandlerAction:

    "A handler action that can be specialised for individual forms."

    def __init__(self, pagename):
        self.pagename = pagename

    def processForm(self, request):

        """
        Interpret the given 'request' details and modify them according to the
        structure of the interpreted information.
        """

        # Get the form fields and obtain the hierarchical field structure.

        form = get_form(request)
        fields = getFields(form, remove=True)

        # Process modification operations.

        self.modifyFields(fields)
        self.serialiseFields(fields, form)

        do_show(self.pagename, request)

    def serialiseFields(self, fields, form, path=None):

        """
        Serialise the given 'fields' to the given 'form', using the given 'path'
        to name the entries.
        """

        for key, value in fields.items():

            # Serialise sections.

            if isinstance(value, dict):
                for index, element in enumerate(getSectionElements(value)):
                    element_ref = "%s$%s" % (key, index)

                    self.serialiseFields(element, form,
                        path and ("%s/%s" % (path, element_ref)) or element_ref
                        )

            # Serialise fields.

            else:
                form[path and ("%s/%s" % (path, key)) or key] = value

    def modifyFields(self, fields):

        "Modify the given 'fields', removing and adding items."

        # First, remove fields.

        for key in fields.keys():
            if key.startswith("_remove="):
                self.removeField(key[8:], fields)

        # Then, add fields.

        for key in fields.keys():
            if key.startswith("_add="):
                self.addField(key[5:], fields)

    def removeField(self, path, fields):

        """
        Remove the section element indicated by the given 'path' from the
        'fields'.
        """

        section, (name, index) = getSectionForPath(path, fields)
        del section[name][index]

    def addField(self, path, fields):

        """
        Add a section element indicated by the given 'path' to the 'fields'.
        """

        section, (name, index) = getSectionForPath(path, fields)
        placeholder = {"_new" : ""}

        if section.has_key(name):
            indexes = section[name].keys()
            max_index = max(map(int, indexes))
            section[name][max_index + 1] = placeholder
        else:
            max_index = -1
            section[name] = {0 : placeholder}

# Action function.

def execute(pagename, request):
    MoinFormHandlerAction(pagename).processForm(request)

# vim: tabstop=4 expandtab shiftwidth=4
